package sir.estimate;

/**
 * Title:        Options object. 
 * Description:  All about the configuration goes here
 * Copyright:    
 * Company:
 * @author Michael Hhle &lt;hoehle@dina.kvl.dk&gt;
 * @version 1.0
 *
 * This class contains options for the SIREstimate program. Could have been implemented in the
 * class itself, but this way it might be more flexible in case integration with the simulation
 * part is done at some point.
 */

public class Options {
  //////////////////////////////////////////////////////////////////////////
  /////////////////////////////// Option Part //////////////////////////////
  //////////////////////////////////////////////////////////////////////////
  public static boolean readFromFile = false;
  
  /** Are we using a random seed value or are we fixing the value to be able
   *  to get same results later on?
   */
  public static boolean fixedSeed = false;

  /** Seed value for the random generator (if required) */
  public static int seed;

  /** Ignore any E specified, i.e. all E should be parametric? */
  public static boolean ignoreE;
  /** Ignore any I specified, i.e. all I should be parametric? */
  public static boolean ignoreI;
  /** Ignore any S specified, i.e. all D should be parametric? */
  public static boolean ignoreD;

  /** Should we use a mean/variance representation of all Gamma distribution? */
  public static boolean meanVar;
  /** Distribution type for incubation period */
  public static char incuTime;
  /** Is the infectious period exponentially distributed? Note: meanVar=T not allowed if true */
  public static boolean expInfTime;
  /** Distribution of the Seroconversion period: 'E'=exponential, 'G'=gamma, 'N'=none (ignore)   */
  public static char diagTime;
  

  /** Which method did we actually pick? Neater solution would be to have an interface
   *  and MCMC, ML, MML as classes each implementing their own run method. */
  public static String method;

  //////////////////////////////////////////////////////////////////////////
  /////////////////////////////// MCMC part ////////////////////////////////
  //////////////////////////////////////////////////////////////////////////
  /** Internal option */
  public static boolean slowButSafe = false;
  /** How many samples should the estimation be based on */
  public static int mcmc_samples;
  /** When generating samples, we only take each thin'th sample - others are thrown away */
  public static int mcmc_thin;
  /** Initial number of burnin samples to generate. Note: No thinning is applied here! */
  public static int mcmc_burnin;
   /** Std. dev of proposal distribution (Gaussian) when suggesting new beta values in the Metropolis Hastings setting.*/
  public static double mcmc_betaRWsigma;
    /* Std. dev of prop. distribution of betaN */
  public static double mcmc_betaNRWsigma;
  /** Std. dev of Gaussian proposal distribution for gammaE */
  public static double mcmc_gammaERWsigma;
  /** Std. dev of Gaussian proposal distribution for deltaE */
  public static double mcmc_deltaERWsigma;
  /** Std. dev of Gaussian proposal distribution for gammaI */
  public static double mcmc_gammaIRWsigma;
  /** Std. dev of Gaussian proposal distribution for deltaI */
  public static double mcmc_deltaIRWsigma;
  /** Std. dev of Gaussian proposal distribution for gammaD */
  public static double mcmc_gammaDRWsigma;
  /** Std. dev of Gaussian proposal distribution for deltaD */
  public static double mcmc_deltaDRWsigma;
  /** Std. dev of Gaussian proposal distribution for R */
  public static double mcmc_RRWsigma = 3;
  /** Std. dev of Gaussian proposal distribution for I */
  public static double mcmc_IRWsigma = 3;
  /** Std. dev of Gaussian proposal distribution for E */
  public static double mcmc_ERWsigma = 3;

  //////////////////////////////////////////////////////////////////////////
  /////////////////////////////// MML  part ////////////////////////////////
  //////////////////////////////////////////////////////////////////////////

  /** How many different samples of \bm{I} is the estimation based on.
   *  Typically one is sufficient, but to check for stability of the MC Integration
   *  try a moderate number.
   */
  public static int mml_samples;
  /** Number of samples to use in the Monte Carlo integration approximation
   *  of the marginal density */
  public static int mml_mcsamples;
  /** Cut off value for sampling */
  public static double mml_theta;
   /** Should the optimizer run silent or print extra information? */
  public static boolean mml_verboseOptimizer;
  /** Scale factor to use when adding the individual monte carlo samples.
   *  Use scaleFactor>1 to get an effect!
   */
  public static double mml_sumscale;

  /** Standard DEBUG routine. Note: Not all parameters are shown. */
  public String toString() {
     StringBuffer s = new StringBuffer();
     s.append("seed=" + seed + "\nmethod = " + method + "\nbetaRWsigma = " + mcmc_betaRWsigma + "\nverboseOptimizer=" + mml_verboseOptimizer);
     return(s.toString());
  }

  public Options() {
  }
}