package sir.sim;

import org.omegahat.Simulation.RandomGenerators.*;
import sir.util.*;
import sir.model.*;
import java.text.NumberFormat;
import java.text.DecimalFormat;
import java.util.*;
import java.io.*;
import sir.estimate.*;

/**
   New sampler to handle Sellke constructions. Currently, only the single homogenous
   setup is handled. Intention is to go to a spatial setup at some point.
   Added seroconversion.

   * Added so vaccine effects etc. can calculate predictive posterior distributions.
   * Now calculates DIC5.
   * 
   @author Michael Hhle, 17 October 2002. Last modified on 14 October 2003.
*/

public class Score {
  /** Debug state */
  boolean DEBUG = false;

  //Identifiers.
  public static int beta_ = 1;
  public static int betaN_ = 2;
  public static int gammaE_ = 3;
  public static int deltaE_ = 4;
  public static int gammaI_ = 5;
  public static int deltaI_ = 6;
  public static int gammaD_ = 7;
  public static int deltaD_ = 8;
  /** Data object storing the information read form the .data file */
  public static Data data;

  /** Distribution object allowing us to sample from various densities etc. */
  public static Distributions prob;

  /** For formatting of doubles to an output stream. */
  //static protected NumberFormat form;
  static protected DecimalFormat form;

      /** Read the specification from the data file. We share the .data specification
   * with the LadyBug program. This works coz the parser sets the necessary variables?
   *
   * @param fileName Name of the .data file...
   */
  public static Data readDataFile(String fileName) {
    System.out.println("---------- Reading data ----------\n");
    Data data = null;
    //Open the file for reading
    try {
      //Create parser and associate file to it.
      DataFileSpec parser = new DataFileSpec(new FileInputStream(fileName));

      try {
        //Do the parsing, i.e. start with first production.
        parser.DataFile();
        data = parser.getData();
        //Make sure our data is valid.
        if (!data.isOk()) {
          System.exit(0);
        }
      }
      catch (ParseException e) {
        System.err.println("Error: Problems while parsing " + fileName + ".");
        System.out.println(e);
        System.exit(0);
      }
    }
    catch (java.io.FileNotFoundException e) {
      System.out.println(e);
      System.exit(0);
    }
    return (data);
  }

  /** Method to initialize the sampler. Move out of
   * the instance class, coz that might mess things up and create
   * non randomness calling that too many times once for each epidemic.
   */
  static public Distributions newSampler() {
    // Seed value for Random number generator (this is a little dirty!)
    java.util.Random r = new java.util.Random();
    int seed = Math.abs(r.nextInt()); //should be 1 - 2^31-1 and notjust abs

    //Fixed seeding - in order to be able to restore results.
    seed = 23132;

    //PRNG generating
    CollingsPRNGAdministrator a = new CollingsPRNGAdministrator(seed);
    PRNG prng = new CollingsPRNG(a.registerPRNGState());

    //Create distribution object.
    return (new Distributions(prng));
  }

  /**
   * Compute Deviance Information criterion for the model. Unfortuntaly we do not have access
   * to the origins. 
   * 
   * @param logFileName Name of the logfile containing the posterior sample of theta and Z
   */
  public void DIC5(String logFileName, PrintStream out) {
    //Update static Datalink in SIRState - NOT VERY NICE
    EpiState.setData(data);
    EpiState.setProb(LadyBug.prob);
    prob = LadyBug.prob;
    
    //A model instance
    SEIRwDState state = new SEIRwDState();
    SEIRwDState mapState = null;//= new SEIRwDState();
    //Sample a state - sample censored events=false
    ((EpiState)state).sampleState(false);
    //((EpiState)mapState).sampleState(false);
    //No origins trick (we don't have the values...)
    state.useOrigins = false;    
    //mapState.useOrigins = false;
    
    //Open the logfile 
    try {
      BufferedReader buf = new BufferedReader(new FileReader(logFileName));
      //Skip header
      String aLine = buf.readLine();

      //Loop over all samples in the file - i.e. read all Lines
      int sampleSize=0;
      double dev=0,loglik=0;
      //To save the maximum.
      //double mapTheta = new double[state.noOfModelParams+1];
      double map=-1e99;
      
      while ( ( (aLine = buf.readLine()) != null)) {
        //Transfer logfile line to parameters
        state.fromString(aLine);
        loglik = state.loglik();
        
        //Only non infinte values of the likelihood are used.
        //Without origins slightly different results are obtained, but
        //why we can have negative logliks should be checked.
    
        if (!Double.isInfinite(loglik)) {
          dev += -2 * loglik;
          
          //calculate log pi(\theta)
          state.theta = state.params2Array();
          double logpi = 0;
          for (int i=1; i<=state.noOfModelParams; i++) {
            //In a 1-unit setup betaN is not part of the model.
            if ( (i != betaN_) || (data.noOfUnits > 1)) {
              logpi +=
                  prob.loggammaPDF(state.theta[i], data.nu[i], data.lambda[i]);
            }
          }
          
          //If we got a new maximum
          if (loglik + logpi > map) {
            System.out.println("loglik=" + loglik + "\tlogpi = " + logpi);
            map = loglik + logpi;
            mapState = (SEIRwDState)state.clone();
           
            //Check      
            //System.out.println("loglik = " + mapState.loglik());     
         }
          sampleSize++;
        }//done infinite checking
      } //done reading lines
      
      //calculate pD
      double devBar = dev/(double)sampleSize;
      System.out.println("Map State=");
      

      System.out.println(mapState.loglik());
      double pD = devBar - (- 2*mapState.loglik());

      System.out.println("Posterior Mean of Deviance = " + devBar );
      System.out.println("pD                         = " + pD );
      System.out.println("DIC5                       = " + (devBar + pD));
      
      out.println("DIC5\t" + devBar + "\t" + pD + "\t" + (devBar+pD));
      //Close file
      buf.close();
    } catch (Exception e) {
      System.err.println(e);
    }
  }
  
  
  /**
   * Compute Deviance Information criterion for the model. 
   * This one samples for each state of Z!
   * 
   * @param logFileName Name of the logfile containing the posterior sample of theta and Z
   * @param out Stream where results are sent.
   */
  public void DIC4(String logFileName, PrintStream out, int burnin, int filter, int sampleSize) {
    //A model instance and sample a state so all arrays are initialiyed
    SEIRwDState state = new SEIRwDState();
    ((EpiState)state).sampleState(false);
    
    SEIRwDState mapState = null;
    //Sample a state - sample censored events=false
    
    //No origins trick (we don't have the values...)
    state.useOrigins = false;    
  
    //Open the logfile 
    try {
      BufferedReader buf = new BufferedReader(new FileReader(logFileName));
      //Skip header
      String aLine = buf.readLine();

      //Loop over all samples in the file - i.e. read all Lines
      int logSampleSize=0;
      double dev=0,loglik=0,loglikEZ=0;
      //To save the maximum.
      //double mapTheta = new double[state.noOfModelParams+1];
      double map=-1e99;

      while ( ( (aLine = buf.readLine()) != null)) {
        
        //Transfer logfile line to parameters
        state.fromString(aLine);
        loglik = state.loglik();

        //Only non infinte values of the likelihood are used.
        //Without origins slightly different results are obtained, but
        //why we can have negative logliks should be checked.
        if (!Double.isInfinite(loglik)) {
          dev += -2 * loglik;

          //Increase line counter
          logSampleSize++;

          //Sample mean of theta while keeping Z={E,I,R,D} fixed.
          //int burnin = 100;
          //int filter = 1;
          //int sampleSize = 100;
          int totalNoOfSamples = burnin + filter * sampleSize;
          double Etheta[] = new double[state.noOfModelParams + 1];
          state.runs = 0;

          //Generate the sample of size sampleSize
          for (int sampleCounter = 1; sampleCounter <= totalNoOfSamples;
               ++sampleCounter) {
            //Do a Gibbs within Metropolis-step - but we just update theta
            state.generateNext(true);
            //Do we pick this value - then add it to theta.
            if ( (sampleCounter > burnin) &&
                ( (sampleCounter - burnin) % filter == 0)) {
              state.params2Array();
              for (int i = 1; i <= state.noOfModelParams; i++) {
                Etheta[i] += 1 / (double) sampleSize * state.theta[i];
              }
            }
          } //done with mcmc analysis

          //Insert the posterior mean of theta into the state object
          state.array2Params(Etheta, false);
          //System.out.println(state.toStringLong());
          loglikEZ += state.loglik();
          //System.out.println(logSampleSize + ": dev=" + form.format(dev) + " loglik=" + form.format(loglik) + " loglikEZ=" + form.format(loglikEZ) + " loglik=" + form.format(state.loglik()));
          System.out.println(logSampleSize + "\t");
        }
      } //done looping over log file.
          
      //calculate pD
      double devBar = dev/(double)logSampleSize;
      double devBarTheta = -2*loglikEZ/(double)logSampleSize;
      double pD = devBar - devBarTheta;

      System.out.println("Posterior Mean of Deviance = " + devBar );
      System.out.println("pD                         = " + pD );
      System.out.println("DIC4                       = " + (devBar + pD));
      
      //Write to logfile
      out.println("DIC4\t" + devBar + "\t" + pD + "\t" + (devBar+pD));

      //Close file
      buf.close();
    } catch (Exception e) {
      System.err.println(e);
    }
  }


  /**
     A sir sampler for command line calling.
   */
  static public void main(String[] argv) {
    //Argv indexes
    int _DATAFILE = 0;
    int _LOGFILE = 1; //Name of the estimate log file.
    int _RESULTS = 2; //FileName of DIC result file
    int _BURNIN = 3;
    int _FILTER = 4;
    int _SAMPLESIZE = 5;
    
    ///////////////////////////////////////////////////////////
    //Initialize - weird global variable setting..
    LadyBug.prob = newSampler();
    Score score = new Score();
    score.prob = LadyBug.prob;
    EpiState.setProb(LadyBug.prob);
    

    //Setup the formatting to US style and 5 digits
    //form = NumberFormat.getInstance(Locale.ENGLISH);
    //form.setMinimumFractionDigits(4);
    //form.setMaximumFractionDigits(4);

    //Initialize some format
    Locale.setDefault(Locale.ENGLISH);
    //form = new DecimalFormat("####.000000");
    form = new DecimalFormat("0.000E00");
    LadyBug.form = form;
    /////////////////////////////////////////////////////////
    
    PrintStream results = null;
    
    try {
      /** Data object storing the information read form the .data file. */
      score.data = score.readDataFile(argv[_DATAFILE]);
      EpiState.setData(data);
      
      //Open logfile
      results = new PrintStream(new FileOutputStream(argv[_RESULTS],false));
      //write header
      results.println("name\tbarD\tpD\tDIC");
      score.DIC4(argv[_LOGFILE],results,Integer.parseInt(argv[_BURNIN]),Integer.parseInt(argv[_FILTER]),
                 Integer.parseInt(argv[_SAMPLESIZE]));
      score.DIC5(argv[_LOGFILE],results);

    }
    catch (ArrayIndexOutOfBoundsException e) {
      System.err.println("Syntax Error:\n\nScore <DATAFILE> <LOGFILE> <RESULTFILE> <BURNIN> <FILTER> <SAMPLESIZE>");
    }
    catch (FileNotFoundException e) {
      System.err.println("Error: Can not open results file \"" + argv[_RESULTS] + "\"");
    }
    finally {
      if (results != null) {results.close();}
    }
  }
}
